#!/usr/bin/env python

# This script runs with both python2 and python3

# Copyright (C) 2014 Paul D Turner & The CEGUI Development Team
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import glob
import sys
import os
import os.path

try:
    from lxml import etree

except ImportError:
    print("WARNING: Failed to import lxml. This most probably means you "
          "haven't got lxml installed. Run `pip install lxml` or "
          "`yum install python-lxml` and then retry.")

    # We intentionally exit with 0, failure to find lxml doesn't mean datafiles
    # are sure to be invalid.
    sys.exit(0)


def validate_file(schema_path, file_path):
    # TODO: Yeah yeah, we load the XMLSchema over and over, who cares...
    schema = etree.XMLSchema(file = schema_path)
    if schema.validate(etree.parse(file_path)):
        return True, []

    else:
        return False, schema.error_log


def gather_files(xml_files, datafiles_path, schema_path, globs):
    for glob_ in globs:
        for file_ in glob.glob(os.path.join(datafiles_path, glob_)):
            xml_files[file_] = os.path.join(datafiles_path, schema_path)


def main():
    xml_files = {}

    datafiles_path = os.path.join(os.path.dirname(__file__), "..", "datafiles")

    gather_files(xml_files, datafiles_path,
                 "xml_schemas/Animation.xsd", ["animations/*.anims"])
    gather_files(xml_files, datafiles_path,
                 "xml_schemas/Font.xsd", ["fonts/*.font"])
    gather_files(xml_files, datafiles_path,
                 "xml_schemas/Imageset.xsd", ["imagesets/*.imageset"])
    gather_files(xml_files, datafiles_path,
                 "xml_schemas/GUILayout.xsd", ["layouts/*.layout", "layouts/*.wnd"])
    gather_files(xml_files, datafiles_path,
                 "xml_schemas/Falagard.xsd", ["looknfeel/*.looknfeel"])
    gather_files(xml_files, datafiles_path,
                 "xml_schemas/GUIScheme.xsd", ["schemes/*.scheme"])

    failed_xml_files = 0

    for file_path in sorted(xml_files):
        schema_path = xml_files[file_path]

        result, error_log = validate_file(schema_path, file_path)

        file_path = os.path.relpath(file_path, datafiles_path)
        schema_path = os.path.relpath(schema_path, datafiles_path)

        if result:
            print("%s is valid against %s" % (file_path, schema_path))

        else:
            print("")
            print("ERROR: %s failed validation against %s" % (file_path, schema_path))
            for error in iter(error_log):
                print(error)

            print("")

            failed_xml_files += 1

    print("")

    if failed_xml_files == 0:
        print("All %i files are valid!" % (len(xml_files)))
        sys.exit(0)

    else:
        print("%i files out of %i (%f %%) failed to validate!" % (failed_xml_files, len(xml_files), 100.0 * float(failed_xml_files) / float(len(xml_files))))
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python2

#/***********************************************************************
#    filename:   generateCEGUIOgreRenderer.py
#    created:    12/11/2010
#    author:     Martin Preisler (with many bits taken from python ogre)
#
#    purpose:    Generates CEGUI Ogre Renderer binding code
#*************************************************************************/
#/***************************************************************************
# *   Copyright (C) 2004 - 2013 Paul D Turner & The CEGUI Development Team
# *
# *   Thanks to Roman Yakovenko for advices and great work on Py++!
# *   Thanks to Andy Miller for his python-ogre CEGUI bindings!
# *
# *   License: generator is GPL3 (python ogre code generators are also GPL)
# *            generated code is MIT as the rest of CEGUI

import os

from pyplusplus import function_transformers as ft
from pyplusplus import messages
from pyplusplus.module_builder import call_policies
from pygccxml import declarations

import common_utils
import cegui_base

PACKAGE_NAME = "PyCEGUIOgreRenderer"
PACKAGE_VERSION = common_utils.GLOBAL_PACKAGE_VERSION
MODULE_NAME = PACKAGE_NAME

OUTPUT_DIR = os.path.join(common_utils.OUTPUT_DIR, "CEGUIOgreRenderer")

def filterDeclarations(mb):
    # by default we exclude everything and only include what we WANT in the module
    mb.global_ns.exclude()

    CEGUI_ns = mb.global_ns.namespace("CEGUI")

    # RendererModules/Ogre/CEGUIOgreRenderer.h
    renderer = CEGUI_ns.class_("OgreRenderer")
    renderer.include()
    renderer.noncopyable = True

    # RendererModules/Ogre/CEGUIOgreResourceProvider.h
    resourceprovider = CEGUI_ns.class_("OgreResourceProvider")
    resourceprovider.include()
    resourceprovider.noncopyable = True

    # RendererModules/Ogre/CEGUIOgreImageCodec.h
    imagecodec = CEGUI_ns.class_("OgreImageCodec")
    imagecodec.include()
    imagecodec.noncopyable = True

    common_utils.excludeAllPrivate(CEGUI_ns)

    # for users using python-ogre in conjunction to PyCEGUI
    mb.global_ns.class_( "::Ogre::RenderTarget" ).already_exposed = True
    mb.global_ns.class_( "::Ogre::TexturePtr" ).already_exposed = True
    mb.global_ns.class_( "::Ogre::Matrix4" ).already_exposed = True

def generate():
    ### disable unnecessary warnings
    # py++ will create a wrapper
    messages.disable(messages.W1023, messages.W1025, messages.W1026, messages.W1027, messages.W1031)
    # can't be overridden in python
    messages.disable(messages.W1049)

    # "CEGUIBASE_EXPORTS" seems to help with internal compiler error with VS2008SP1 and gccxml 0.9
    mb = common_utils.createModuleBuilder("python_CEGUIOgreRenderer.h", ["OGRE_GUIRENDERER_EXPORTS", "CEGUIBASE_EXPORTS"])
    CEGUI_ns = mb.global_ns.namespace("CEGUI")

    mb.register_module_dependency(cegui_base.OUTPUT_DIR)

    common_utils.addSupportForString(mb)

    filterDeclarations(mb)

    common_utils.setDefaultCallPolicies(CEGUI_ns)

    ## add additional version information to the module to help identify it correctly 
    common_utils.addVersionInfo(mb, PACKAGE_NAME, PACKAGE_VERSION)

    # Creating code creator. After this step you should not modify/customize declarations.
    mb.build_code_creator(module_name = MODULE_NAME, doc_extractor = common_utils.createDocumentationExtractor())

    common_utils.writeModule(mb, OUTPUT_DIR)

if __name__ == "__main__":
    common_utils.verbose_generate("CEGUIOgreRenderer", generate)



#!/usr/bin/env python2

#/***********************************************************************
#    filename:   generate.py
#    created:    13/8/2010
#    author:     Martin Preisler (with many bits taken from python ogre)
#
#    purpose:    Generates all CEGUI related python bindings
#*************************************************************************/
#/***************************************************************************
# *   Copyright (C) 2004 - 2013 Paul D Turner & The CEGUI Development Team
# *
# *   Thanks to Roman Yakovenko for advices and great work on Py++!
# *   Thanks to Andy Miller for his python-ogre CEGUI bindings!
# *
# *   License: generator is GPL3 (python ogre code generators are also GPL)
# *            generated code is MIT as the rest of CEGUI

import generators.cegui_base

import generators.opengl_renderer
import generators.ogre_renderer
import generators.null_renderer

from generators.common_utils import verbose_generate
import os

if __name__ == "__main__":
    verbose_generate("CEGUIBase", generators.cegui_base.generate)

    verbose_generate("CEGUINullRenderer", generators.null_renderer.generate)
    verbose_generate("CEGUIOgreRenderer", generators.ogre_renderer.generate)
    verbose_generate("CEGUIOpenGLRenderer", generators.opengl_renderer.generate)



#!/usr/bin/env python2

#/***********************************************************************
#    filename:   generateCEGUIOpenGLRenderer.py
#    created:    13/8/2010
#    author:     Martin Preisler (with many bits taken from python ogre)
#
#    purpose:    Generates CEGUI OpenGL Renderer binding code
#*************************************************************************/
#/***************************************************************************
# *   Copyright (C) 2004 - 2013 Paul D Turner & The CEGUI Development Team
# *
# *   Thanks to Roman Yakovenko for advices and great work on Py++!
# *   Thanks to Andy Miller for his python-ogre CEGUI bindings!
# *
# *   License: generator is GPL3 (python ogre code generators are also GPL)
# *            generated code is MIT as the rest of CEGUI

import os

from pyplusplus import function_transformers as ft
from pyplusplus import messages
from pyplusplus.module_builder import call_policies
from pygccxml import declarations

import common_utils
import cegui_base

PACKAGE_NAME = "PyCEGUIOpenGLRenderer"
PACKAGE_VERSION = common_utils.GLOBAL_PACKAGE_VERSION
MODULE_NAME = PACKAGE_NAME

OUTPUT_DIR = os.path.join(common_utils.OUTPUT_DIR, "CEGUIOpenGLRenderer")

def filterDeclarations(mb):
    # by default we exclude everything and only include what we WANT in the module
    mb.global_ns.exclude()

    CEGUI_ns = mb.global_ns.namespace("CEGUI")

    # RendererModules/OpenGL/RendererBase.h
    rendererBase = CEGUI_ns.class_("OpenGLRendererBase")
    rendererBase.include()
    rendererBase.mem_fun("setViewProjectionMatrix").exclude()
    rendererBase.mem_fun("getViewProjectionMatrix").exclude()
    rendererBase.noncopyable = True

    # RendererModules/OpenGL/GLRenderer.h
    renderer = CEGUI_ns.class_("OpenGLRenderer")
    renderer.include()
    renderer.mem_fun("setViewProjectionMatrix").exclude() # CEGUI::mat4Pimpl
    renderer.noncopyable = True

    # RendererModules/OpenGL/GeometryBufferBase.h
    geometryBufferBase = CEGUI_ns.class_("OpenGLGeometryBufferBase")
    geometryBufferBase.include()
    geometryBufferBase.mem_fun("getMatrix").exclude() # CEGUI::mat4Pimpl
    geometryBufferBase.noncopyable = True

    # RendererModules/OpenGL/OpenGLTextureTarget.h
    textureTarget = CEGUI_ns.class_("OpenGLTextureTarget")
    textureTarget.include()
    textureTarget.noncopyable = True

    # RendererModules/OpenGL/OpenGLViewportTarget.h
    viewportTarget = CEGUI_ns.class_("OpenGLViewportTarget")
    viewportTarget.include()
    viewportTarget.noncopyable = True

    # RendererModules/OpenGL/GL3Renderer.h
    renderer3 = CEGUI_ns.class_("OpenGL3Renderer")
    renderer3.include()
    renderer3.noncopyable = True

    # RendererModules/OpenGL/Shader.h
    shader3 = CEGUI_ns.class_("OpenGL3Shader")
    shader3.include()
    shader3.noncopyable = True

    # RendererModules/OpenGL/StateChangeWrapper.h
    stateChangeWrapper3 = CEGUI_ns.class_("OpenGL3StateChangeWrapper")
    stateChangeWrapper3.include()
    stateChangeWrapper3.noncopyable = True

def generate():
    ### disable unnecessary warnings
    # py++ will create a wrapper
    messages.disable(messages.W1023, messages.W1025, messages.W1026, messages.W1027, messages.W1031)
    # can't be overridden in python
    messages.disable(messages.W1049)

    # "CEGUIBASE_EXPORTS" seems to help with internal compiler error with VS2008SP1 and gccxml 0.9
    mb = common_utils.createModuleBuilder("python_CEGUIOpenGLRenderer.h", ["OPENGL_GUIRENDERER_EXPORTS", "CEGUIBASE_EXPORTS"])
    CEGUI_ns = mb.global_ns.namespace("CEGUI")

    mb.register_module_dependency(cegui_base.OUTPUT_DIR)

    common_utils.addSupportForString(mb)

    filterDeclarations(mb)

    common_utils.setDefaultCallPolicies(CEGUI_ns)

    ## add additional version information to the module to help identify it correctly 
    # todo: this should be done automatically
    common_utils.addVersionInfo(mb, PACKAGE_NAME, PACKAGE_VERSION)

    # Creating code creator. After this step you should not modify/customize declarations.
    mb.build_code_creator(module_name = MODULE_NAME, doc_extractor = common_utils.createDocumentationExtractor())

    common_utils.writeModule(mb, OUTPUT_DIR)

if __name__ == "__main__":
    common_utils.verbose_generate("CEGUIOpenGLRenderer", generate)



# This file has been generated by Py++.

# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0. (See
# accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

class named_tuple(tuple):
    """Creates tuple, which allows access to stored values by name and\or by index.
    
    Construction example: named_tuple( ('a',0), ('b',1) )
        'a' and 'b' are names, while 0 and 1 are values
    """

    def __new__(cls, *args):
        return tuple.__new__( cls, [ val for name, val in args] )

    def __init__(self, *args):
        tuple.__init__( self )
        self.__dict__[ '__name2value' ] = dict( args )

    def __getattr__(self, name):
        try:
            return self.__dict__['__name2value'][ name ]        
        except KeyError:
            raise AttributeError( "named_tuple has no attribute '%s'" % name )

    def __setattr__(self, name, value):
        raise AttributeError( "named_tuple has no attribute '%s'" % name )

    def __getitem__( self, key ):
        #TODO: it could be nice to support slicing. So the __getitem__ in case of 
        #slicing will return new named_tuple.
        if isinstance( key, basestring ):
            return self.__dict__['__name2value'][ key ]        
        else:
            return super( named_tuple, self ).__getitem__( key )

if __name__ == '__main__':
    nt = named_tuple( ('a',0), ('b',1) )
    assert nt.a == 0 and nt.b == 1
    a,b = nt
    assert a == 0 and b == 1
    assert nt[ "a" ] == 0 and nt[ "b" ] == 1
    




#/***********************************************************************
#    filename:   common_utils.py
#    created:    13/8/2010
#    author:     Martin Preisler (with many bits taken from python ogre)
#
#    purpose:    Groups some otherwise repeated code
#*************************************************************************/
#/***************************************************************************
# *   Copyright (C) 2004 - 2013 Paul D Turner & The CEGUI Development Team
# *
# *   Thanks to Roman Yakovenko for advices and great work on Py++!
# *   Thanks to Andy Miller for his python-ogre CEGUI bindings!
# *
# *   License: generator is GPL3 (python ogre code generators are also GPL)
# *            generated code is MIT as the rest of CEGUI

import os, sys, time

from pyplusplus import module_builder
from pyplusplus.module_builder import call_policies
from pygccxml import declarations

import extract_documentation as exdoc

def get_root_base_path():
    """Returns directory the source tarball was extracted to.
    """

    def ancestor_dir(start_dir, distance = 1):
        cur_dir = start_dir
        for _ in range(distance):
            cur_dir = os.path.dirname(cur_dir)

        return cur_dir

    file_dir = os.path.dirname(__file__)
    return ancestor_dir(file_dir, 6)

GLOBAL_PACKAGE_VERSION = "0.8.0"
GCCXML_PATH = "C:\\Users\\Martin Preisler\\Devel\\PythonPackages\\gccxml_bin\\v09\\win32\\bin\\"
ROOT_BASE_PATH = get_root_base_path()
OUTPUT_DIR = os.path.join(ROOT_BASE_PATH, "cegui", "src", "ScriptModules", "Python", "bindings", "output")
INCLUDE_PATHS = [
    os.path.join(ROOT_BASE_PATH, "cegui", "include"),
    os.path.join(ROOT_BASE_PATH, "build", "cegui", "include") # the usual build include dir
]
BINDINGS_HEADER_PATH = os.path.relpath(os.path.join(ROOT_BASE_PATH, "cegui", "src", "ScriptModules", "Python", "bindings", "generators", "include"))

def createModuleBuilder(input_file, defined_symbols):
    ret = module_builder.module_builder_t(
        files = [
            os.path.join(BINDINGS_HEADER_PATH, input_file)
        ],
        gccxml_path = GCCXML_PATH,
        include_paths = INCLUDE_PATHS,
        define_symbols = defined_symbols,
        indexing_suite_version = 2
    )

    ret.BOOST_PYTHON_MAX_ARITY = 25
    ret.classes().always_expose_using_scope = True

    return ret

def addConstants(mb, constants):
    tmpl = 'boost::python::scope().attr("%(name)s") = %(value)s;'
    for name, value in constants.items():
        mb.add_registration_code(tmpl % dict(name = name, value = str(value)))

def addVersionInfo(mb, name, version):
    import datetime

    t = datetime.datetime.now().isoformat(' ').strip()

    docstring = "%s - version %s" % (name, version)

    addConstants(mb, {
                    'CompileTime__' : '__TIME__',
                    'CompileDate__' : '__DATE__',
                    'Version__' : '"%s"' % version.replace("\n", "\\\n" ),
                    '__doc__' : '"%s"' % docstring.replace("\n", "\\\n" )
                    })

def setDefaultCallPolicies(ns):
    # Set the default policy to deal with pointer/reference return types to reference_existing object
    # as this is the CEGUI Default.
    mem_funs = ns.calldefs()
    mem_funs.create_with_signature = True #Generated code will not compile on
    #MSVC 7.1 if function has throw modifier.
    for mem_fun in mem_funs:
        if mem_fun.call_policies:
            continue
        if declarations.is_pointer (mem_fun.return_type) or declarations.is_reference (mem_fun.return_type):
            mem_fun.call_policies = call_policies.return_value_policy(call_policies.reference_existing_object)

def createDocumentationExtractor():
    return exdoc.doc_extractor("")

def writeModule(mb, output_dir):
    mb.code_creator.user_defined_directories.append(output_dir)
    mb.split_module(output_dir)

def excludeAllPrivate(cls):
    cls.decls(declarations.matchers.access_type_matcher_t("private")).exclude()

    # include back pure virtual private functions
    query = declarations.matchers.access_type_matcher_t("private") \
            & declarations.virtuality_type_matcher_t(declarations.VIRTUALITY_TYPES.PURE_VIRTUAL)

    cls.mem_funs(query, allow_empty = True).include()

# this adds string converters - should only be used in CEGUI package!
def addStringConverters(mb):
    mb.add_declaration_code(
"""
struct CEGUI_String_to_python : public boost::python::converter::expected_from_python_type<CEGUI::String>
{
    static PyObject* convert(const CEGUI::String& s)
    {
        // use native byteorder
        int byteorder = 0;

        // "replace" replaces invalid utf32 chars with "?"
        
        // python wants the size of the buffer, not length of the string,
        // this is the reason for the sizeof
        return boost::python::incref(
            PyUnicode_DecodeUTF32((const char*)(s.ptr()), s.length() * sizeof(CEGUI::utf32), "replace", &byteorder)
        );
    }
};

struct CEGUI_String_from_python
{
    CEGUI_String_from_python()
    {
      boost::python::converter::registry::push_back(
        &convertible,
        &construct,
        boost::python::type_id<CEGUI::String>(),
        &boost::python::converter::expected_from_python_type<CEGUI::String>::get_pytype);
    }
    
    static void* convertible(PyObject* obj_ptr)
    {
        // we allow str() and unicode() objects to be converted to CEGUI::String
        // NOTE: Python3 only has unicode strings
        if (PyUnicode_Check(obj_ptr)
#if PY_MAJOR_VERSION < 3
            || PyString_Check(obj_ptr)
#endif
        )
        {
            return obj_ptr;
        }
        return 0;
    }
    
    static void construct(
      PyObject* obj_ptr,
      boost::python::converter::rvalue_from_python_stage1_data* data)
    {
#if PY_MAJOR_VERSION < 3
        if (!PyUnicode_Check(obj_ptr))
        {
            // no need for unicode, this is just plain str()
            const char* value = PyString_AsString(obj_ptr);
            if (value == 0)
                boost::python::throw_error_already_set();

            void* storage = ((boost::python::converter::rvalue_from_python_storage<CEGUI::String>*)data)->storage.bytes;
            new (storage) CEGUI::String(value);
            data->convertible = storage;
        }
        else
#endif
        {
            // we have to employ a bit more machinery since this is utf16 coded string
            // we encode given unicode object (ucs2 - utf16) to utf8
            PyObject* utf8 = PyUnicode_AsUTF8String(obj_ptr);
            // then we get the raw bytes of the utf8 python object
#if PY_MAJOR_VERSION < 3
            const CEGUI::utf8* value = (CEGUI::utf8*)PyString_AsString(utf8);
#else
            const CEGUI::utf8* value = (CEGUI::utf8*)PyBytes_AsString(utf8);
#endif
            if (value == 0)
                boost::python::throw_error_already_set();

            void* storage = ((boost::python::converter::rvalue_from_python_storage<CEGUI::String>*)data)->storage.bytes;
            new (storage) CEGUI::String(value);
            data->convertible = storage;
            // now we don't need utf8 anymore, CEGUI::String holds the data for us now, by decreasing
            // refcount to utf8, we are practically deallocating it
            boost::python::decref(utf8);
        }
    }
};
"""
    )

    mb.add_registration_code(
"""
boost::python::to_python_converter<
    CEGUI::String,
    CEGUI_String_to_python,
    true>();

CEGUI_String_from_python();
""", tail = False # force converters to be there first
    )

def addSupportForString(mb):
    # set String as exposed, because we provide converters
    mb.global_ns.namespace("CEGUI").class_("String").already_exposed = True

    # we have to set use_make_functions to True for all variables that have CEGUI::String type
    # converters are only used when using setter / getter functions

    for var in mb.global_ns.variables():
        # we don't check the type directly, because a const could be at the end
        # this could be replaced with var.type == "CEGUI::String" or var.type == "CEGUI::String const"
        if str(var.type).startswith("CEGUI::String"):
            var.use_make_functions = True
            # we have to use return_by_value to make converters work
            var.set_getter_call_policies(call_policies.return_value_policy(call_policies.return_by_value))

def verbose_generate(name, generator):
    print("Generating %s bindings..." % (name))
    start_time = time.clock()
    generator()
    print("Finished %s bindings. (took %g seconds)\n" % (name, (time.clock() - start_time)))



"""
extracting from C++ doxygen documented file
Author G.D.

Updates for Ogre handling by AJM - added remarks/note/par
jan 03 06 - fix bug, replace single leading \ with \\ in clear_str
"""

# Taken verbatim from python ogre project

class doc_extractor:
    """
    extracts doxigen styled documentation from source
    or generates from description
    """
    def __init__(self, startswith="", outfile="documentation.dox"):
        #for caching source
        self.file_name  = None
        self.source = None
        self.startswith = startswith
#         self.outfile = open ( outfile, 'w' ) # create the file for writing (truncates existing)
        
    def __call__(self, declaration):
# # #         if not self.startswith in declaration.decl_string:
# #         if not declaration.demangled.startswith(self.startswith):
# #             print "DOC: not in scope", declaration.decl_string 
# #             print "name:", declaration.name
# #             print "demangled:", declaration.demangled
# # #             print "type:", declaration.type
# # #             if "Ogre" in declaration.decl_string : sys.exit()
# #             return ""
        try:    # some types such as the base namespace don't have 'location'
            if self.file_name != declaration.location.file_name:            
                self.file_name = declaration.location.file_name
                self.source = open(declaration.location.file_name).readlines()
        except:
            print "DOC without location:", declaration
            return ""
            
        find_block_end = False
        failed = 0
        doc_lines = []
        if declaration.documentation:
            if len(declaration.documentation) > 2:
               # remove any exisiting quotes and and a new line
               basedoc = declaration.documentation.strip('"') + "\\\n"
        else:
            basedoc = ""
#         print declaration            
#         print "BASEDOC", basedoc            
        ## note the gccxml uses a 1 based line scheme whereas we are using python arrays - 0 based
        ## so we need to subtract 1 from the line number.
        
        ## then we need to subtract another 1 to put us on the line above the function/class etc defination
        
        ## AND to make it worse, Ogre coding style often has the opening brace on the line AFTER the defination name
        ## and gccxml gives us the line number with the brace - so we need to look back even further
        
        ## lets look for a single comment '///'  We'll try upto 2 lines above
        try:
            for lcount in xrange(declaration.location.line-1, declaration.location.line-4 , -1):
                line = self.source[lcount]
                if line.lstrip()[:3] == '///':
                    str_ = clear_str(line)
                    if basedoc:
                        retvalue = '"' + basedoc +  str_[:-2] + '"' # remove the \n from the single line comment
                    else:
                        retvalue = '"' + str_[:-2] + '"' # remove the \n from the single line comment
#                     print "Extracted Doc String (short) ", retvalue
                    
                    return retvalue
                    
            ## if that didn't work we'll look for a block of comments
            
            for lcount in xrange(declaration.location.line - 1, -1, -1):
                line = self.source[lcount]
                #print "Checking ", lcount, line
                if not find_block_end:
                    try:
                        if line.rstrip()[-2:] == "*/":
                            find_block_end = True
                    except:
                        pass
                if  find_block_end:
                    try:
                        if line.lstrip()[:2] == "/*":
                            find_block_end = False
                    except:
                        pass
                final_str = clear_str(line)
                if not find_block_end and code(line):
                    ## note that ogre code tends to have the opening function/class bracket on the 'next' line 
                    ## from the 'name' -so we may need to backup one more line to fine doc string..
                    failed +=1
                    final_str = ""  ## make sure we don't add the extra code line
                    if failed > 2:
                        break
                if final_str:
                    doc_lines.insert(0, clear_str(final_str))
        except:
            if not basedoc:
                return ""
        ret ="" 
        if doc_lines:
            doc_lines = removeBlankLines ( doc_lines )
            doc_lines = removeLeadingSpace ( doc_lines )
            doc_lines = wrapLines ( doc_lines )
#             print "Extracted Doc String for:",  declaration, "[", len(doc_lines),"]"
            ## we need to cope with strings longer than 2048 for MSVC
            ret =  "\\\n".join(doc_lines) 
            if len ( ret ) > 1700:  ## just to be safe and adjust for line end chars etc..
                ret = ""
                length = 1
                for line in doc_lines:
                    ret = ret + line
                    length += len ( line )
                    if length < 1500 :  # again lets be conservative
                        ret = ret + "\\\n"
                    else :
                        ret = ret + '"' + "\n" + '"'    # we close the original 'quote', move to a new line and open a new quote
                        length = 1
        ## Py++ doesn't like non ascii characters in the doc string
        ## class_declaration.py - _generate_constructor - return ( ''.join( result ), used_init )
        newret = ""
#         print "DOC:  GOT SOMETHING", ret
        for c in ret:
            if ord(c) >127:
                c = ' '
            newret = newret + c
        if len(basedoc) >1 or len(newret)>1:
# #             self.outfile.write("***============================*****\n" + declaration.decl_string + "\n")
# #             self.outfile.write( str(len(basedoc)) + "  " + str(len(newret)) + "\n" )
# #             self.outfile.write( newret )      
            retval ='"' + basedoc + newret.lstrip() + '"' 
#             print "RETURNING:", retval
            return retval             
        else: 
            return ""

#===============================================================
def removeBlankLines ( docin ):
    """
    remove leading and trailing blank lines 
    """ 
    # first remove leading blank lines
    x = 0 
    while x <  len ( docin ) - 1 :
        if docin[x].strip() == "\\n" or docin[x].strip() == '':
            del docin[x]
            x = x - 1
        else:
            break
        x += 1
        
    # now from the bottom up    
    x = len ( docin ) -1  
    while x >= 0:
        if docin[x].strip() == "\\n":
            del docin[x]
        else:
            break
        x = len ( docin )- 1
                            
    return docin
                    
#===============================================================
def removeLeadingSpace ( docin ):
    """
    remove leading spaces/tabs to bring minimize space usage
    """
    import string as string
    c = 0
    for x in range ( len (docin) ):
        docin[x] = docin[x].expandtabs(3) # replace tab characters with 3 spaces
    
    remove = -1  # use to track number of leading spaces to remove. -1 indicates first time through     
    for x in range ( len (docin) ):
        c = 0
        for y in range ( len ( docin[x]) ): # now lets count any leading spaces
            if docin[x][y] in string.whitespace:
                c +=1
            else:
                break
        if c > 0:   # Ok so we found leading space
            if remove < 0: # must be first time through
                remove = c
            else:
                if c < remove:  # ensure we get the lowest common value
                    remove = c
        else:   # Ok we found a line without any leading spaces so it's all off
            remove = -1
            break
    if remove > 0: # lets do it again and remove a set of leading spaces
        for x in range ( len (docin) ):
            docin[x] = docin[x][remove:]
    return docin            
    
#===============================================================
def wrapLines ( docin, maxlen = 100 ):
    """
    Break line length to a max while retaining the leading spaces...
    """
    import textwrap as textwrap
    import string
    docout = []
    for line in docin:
        prefix=0
        for c in line:
            if c in string.whitespace:
                prefix += 1
            else:
                break
        for l in textwrap.wrap( line, maxlen ):
            line = ' '*prefix + l.lstrip()
            docout.append ( line )
    return docout    
   
    
def code(str):
    """
    detect str is code?
    """
    try:
        beg = str.lstrip()[:2]
        return  beg != "//" and beg != "/*"
    except:
        pass
    return False

           
def clear_str(_str):
    """
    replace */! by Space and \breaf, \fn, \param, ...
    """
    def clean ( _str, sym, change2 = ""): 
        return _str.replace(sym, change2)
        
        
    _str = clean(_str, "::", ".")     ## make it more python accurate 
    _str = clean(_str, "->", ".") 
#     _str = clean(_str, "@param", "Param: ")
    _str = clean(_str, "\\param", "@param")
    _str = clean(_str, "@ingroup", "@group")
    _str = clean(_str, "\\ingroup", "@group")
#     _str = clean(_str, "@return", "It return")
    _str = clean(_str, "\\return", "@return")
    _str = clean(_str, "\\Return", "@return")
#     _str = clean(_str, "@note", "Note: ")
    _str = clean(_str, "@remarks", "@see")
    _str = clean(_str, "\\remarks", "@see")
    _str = clean(_str, "\\Remarks", "@see")
#     _str = clean(_str, "@see", "See: ")
    _str = clean(_str, "\\see", "@see")
    _str = clean(_str, "@ref", "@see")
    _str = clean(_str, "\\ref", "@see")
    _str = clean(_str, "@copydoc", "Ref: ")
    _str = clean(_str, "\\verbatim", "" )
    _str = clean(_str, "\\endverbatim", "" )
    
    
    _str = clean(_str, "\\sa", "@see")   # comment _string in OgreNewt
    _str = clean(_str, "\\codeblock", "::")    
    _str = clean(_str, "\\code", "::")    
    _str = clean(_str, "\\endcode", "\\n")    
    _str = clean(_str, "@codeblock", "\\n")   # endsure a new line to end the code block 
    _str = clean(_str, "@code", "::")   # using literal code blocks 
    
#     _str = clean(_str, "@par", "")    ## it will get a single blank line by default -- breaks @param...
    _str = clean(_str, "\r\n", "\\n")   ## for linux build
    _str = clean(_str, "\\par", "")    ## it will get a single blank line by default
    _str = clean(_str, "\n", "\\n") 
    _str = clean(_str, "\\p", "")     ## cegui comments
    _str = clean(_str, "\\li", "  * ")     ## ode comments
    _str = clean(_str, "\\exception", "@exception") 
    _str = clean(_str, "\\a", "")     ## cegui
    
    _str = clean(_str, "\\warning", "@Warning: ")     ## nxPhysics
    _str = clean(_str, "\\include", "Include: ")     ## nxPhysics
    _str = clean(_str, "\\image", "Image: ")     ## nxPhysics
    
    _str = clean(_str, "\\examples", "Examples: ")     ## nxOgre
    _str = clean(_str, "\\default", "Default: ")     ## nxOgre

    _str = clean(_str, "\\version", "@version ")     ## cegui
    _str = clean(_str, "\\date", "@date ")     #
    _str = clean(_str, "\\todo", "@todo ")     ## 
    _str = clean(_str, "\\bug", "@Bug ")     ## 
    _str = clean(_str, "\\author", "@Author ")     ## 
    
    
    ## now clean up the rest
    _str = reduce(clean, [_str, '/', "\\brief", '\\fn',
         "@brief", "@fn", '"', "@{", "\\c", "\\a"]) ## somtimes there are '"' in the doc strings and other "\\"...
#     return _str.lstrip()
#     return "  " + _str.lstrip()

    # another fix to remove stray escape sequences - only new lines allowed
    # last character should never be a backslash, however lets fix it just in case...
    # and we need to make a new copy because strings are immutable...
    newStr=""
    for x in range ( len ( _str) -1  ): # handle upto 2nd to last character
        if _str[x] == '\\' and _str[x+1] != 'n':
# #             print "fixing stray escape sequence", x, _str
            newStr += ' ' 
        else:
            newStr += _str[x]                   
    x = len (_str) -1 # need to handle last character
    if _str[x] == '\\' :
# #         print "fixing stray escape sequence EOL", _str
        newStr += ' ' 
    else:
        newStr += _str[x]                   
    return newStr  ## no removale of white space so Epytext can handle sections etc

    
def make_epytext ( str ):
    """ turn this string into something more Epytext compatible """
    knownfields = ['param']
    
if __name__ == '__main__':
    class loc:
        def __init__(self, f, l):
            self.file_name = f
            self.line = l
            
    class x_decl:
        def __init__(self, str, file_name, line):
            self.str  = str
            self.location = loc(file_name, line)
            self.decl_string = ""
            
        def __str__(self):
            return self.str
        
    print doc_extractor("")(x_decl("myfunc(int x, int y)","c:/development/ocvs/ogrenew/ogremain/include/OgreSceneManager.h",218))
    print doc_extractor("")(x_decl("","c:/development/ocvs/ogrenew/ogremain/include/OgreSceneManager.h",223))
    
    print doc_extractor("")(x_decl("","c:/development/CEGUI-0.5.0/include/CEGUIEvent.h",139))



import os
import os.path

# atrocious and unholy!
def get_my_path():
    import fake
    return os.path.dirname(os.path.abspath(fake.__file__))

libpath = get_my_path()
#print "libpath =", libpath
os.environ['PATH'] = libpath + ";" + os.environ['PATH']

from PyCEGUI import *



#!/usr/bin/env python2

#/***********************************************************************
#    filename:   generateCEGUI.py
#    created:    13/8/2010
#    author:     Martin Preisler (with many bits taken from python ogre)
#
#    purpose:    Generates CEGUI binding code
#*************************************************************************/
#/***************************************************************************
# *   Copyright (C) 2004 - 2013 Paul D Turner & The CEGUI Development Team
# *
# *   Thanks to Roman Yakovenko for advices and great work on Py++!
# *   Thanks to Andy Miller for his python-ogre CEGUI bindings!
# *
# *   License: generator is GPL3 (python ogre code generators are also GPL)
# *            generated code is MIT as the rest of CEGUI

import os

from pyplusplus import function_transformers as ft
from pyplusplus import messages
from pyplusplus.module_builder import call_policies
from pygccxml import declarations

import common_utils

PACKAGE_NAME = "PyCEGUI"
PACKAGE_VERSION = common_utils.GLOBAL_PACKAGE_VERSION
MODULE_NAME = PACKAGE_NAME

OUTPUT_DIR = os.path.join(common_utils.OUTPUT_DIR, "CEGUI")

def filterDeclarations(mb):
    # by default we exclude everything and only include what we WANT in the module
    mb.global_ns.exclude()

    # because of std::pair<float, float> CEGUI::Thumb::getHorzRange()
    mb.global_ns.namespace("std").class_("pair<float, float>").include()
    mb.global_ns.namespace("std").class_("pair<CEGUI::String, CEGUI::String>").include()
    mb.global_ns.namespace("std").class_("pair<CEGUI::Image*, CEGUI::ImageFactory*>").include()

    CEGUI_ns = mb.global_ns.namespace("CEGUI")

    ### CORE ###

    # Affector.h
    affector = CEGUI_ns.class_("Affector")
    affector.include()

    # Animation.h
    animation = CEGUI_ns.class_("Animation")
    animation.include()

    # Animation_xmlHandler.h
    # not interesting for python users

    # AnimationInstance.h
    animationInstance = CEGUI_ns.class_("AnimationInstance")
    animationInstance.include()

    # AnimationManager.h
    animationManager = CEGUI_ns.class_("AnimationManager")
    animationManager.include()

    # Base.h
    # nothing interesting for python users
    # todo: maybe pixel align?

    # BasicImage.h
    basicImage = CEGUI_ns.class_("BasicImage")
    basicImage.include()

    # BasicInterpolators.h
    # nothing interesting for python users

    # BasicRenderedStringParser.h
    # todo: is this interesting for python?
    basicRenderedStringParser = CEGUI_ns.class_("BasicRenderedStringParser")
    basicRenderedStringParser.include()

    # BiDiVisualMapping.h
    # Not exposed since this might be disabled at configure time
    #CEGUI_ns.enum("BidiCharType").include()
    #bidiVisualMapping = CEGUI_ns.class_("BidiVisualMapping")
    #bidiVisualMapping.include()

    # BoundSlot.h
    boundSlot = CEGUI_ns.class_("BoundSlot")
    boundSlot.include()
    # also include ref counted variant
    CEGUI_ns.class_("RefCounted<CEGUI::BoundSlot>").include()

    # CentredRenderedString.h
    # todo: is this interesting for python?
    centredRenderedString = CEGUI_ns.class_("CentredRenderedString")
    centredRenderedString.include()

    # ChainedXMLHandler.h
    #chainedXMLHandler = CEGUI_ns.class_("ChainedXMLHandler")
    #chainedXMLHandler.include()

    # Clipboard.h
    #nativeClipboardProvider = CEGUI_ns.class_("NativeClipboardProvider")
    #nativeClipboardProvider.include()
    clipboard = CEGUI_ns.class_("Clipboard")
    clipboard.include()
    # we don't allow native clipboard subclassing/setting from python (for now?)
    clipboard.mem_fun("setNativeProvider").exclude()
    clipboard.mem_fun("getNativeProvider").exclude()

    # exclude methods that are hard to use from within python
    clipboard.mem_fun("setData").exclude()
    clipboard.mem_fun("getData").exclude()

    # Colour.h
    colour = CEGUI_ns.class_("Colour")
    colour.include()

    # ColourRect.h
    colourRect = CEGUI_ns.class_("ColourRect")
    colourRect.include()

    # Config.h
    # nothing interesting for python

    # Config_xmlHandler.h
    # nothing interesting for python

    # CoordConverter.h
    coordConverter = CEGUI_ns.class_("CoordConverter")
    coordConverter.include()

    # DataContainer.h
    rawDataContainer = CEGUI_ns.class_("RawDataContainer")
    rawDataContainer.include()
    rawDataContainer.mem_funs().exclude()

    # DefaultLogger.h
    defaultLogger = CEGUI_ns.class_("DefaultLogger")
    defaultLogger.include()

    # DefaultRenderedStringParser.h
    # not interesting for python

    # DefaultResourceProvider.h
    defaultResourceProvider = CEGUI_ns.class_("DefaultResourceProvider")
    defaultResourceProvider.include()

    # DynamicModule.h
    # not doable in python

    # /Element.h
    element = CEGUI_ns.class_("Element")
    element.include()
    element.class_("CachedRectf").constructors().exclude()

    # Event.h
    event = CEGUI_ns.class_("Event")
    event.include()

    # EventArgs.h
    # handled separately, all classes ending with "EventArgs" are included

    # EventSet.h
    eventSet = CEGUI_ns.class_("EventSet")
    eventSet.include()
    # this is done via custom code
    eventSet.mem_funs("subscribeEvent").exclude()
    eventSet.noncopyable = True

    # we use depth first to make sure classes "deep in the hierarchy" gets their casts tried first
    def collectEventArgsDerived(node):
        ret = []
        for derived in sorted(node.derived):
            ret.extend(collectEventArgsDerived(derived.related_class))

        ret.append(node)

        return ret

    eventArgsCastCode = ""
    for derived in collectEventArgsDerived(CEGUI_ns.class_("EventArgs")):
        eventArgsCastCode += "        if (dynamic_cast<const CEGUI::%s*>(&args))\n" % (derived.name)
        eventArgsCastCode += "        {\n"
        eventArgsCastCode += "            return boost::python::call<bool>(d_callable, static_cast<const CEGUI::%s&>(args));\n" % (derived.name)
        eventArgsCastCode += "        }\n\n"

    eventSet.add_declaration_code(
"""
class PythonEventSubscription
{
public:
    PythonEventSubscription(PyObject* callable):
        d_callable(boost::python::incref(callable))
    {}

    PythonEventSubscription(const PythonEventSubscription& other):
        d_callable(boost::python::incref(other.d_callable))
    {}

    ~PythonEventSubscription()
    {
        boost::python::decref(d_callable);
    }

    bool operator() (const CEGUI::EventArgs& args) const
    {
        // FIXME: We have to cast, otherwise only base class gets to python!

        // I don't understand why this is happening, I think boost::python should use typeid(args).name() and deduce that it's a
        // derived class, not CEGUI::EventArgs base class
        // However this is not happening so I have to go through all EventArgs classes and try casting one after another
""" + eventArgsCastCode + """

        // We could not cast to anything!
        return false;
    }

    PyObject* d_callable;
};

class PythonEventConnection
{
public:
    PythonEventConnection():
        d_connection()
    {}

    PythonEventConnection(const CEGUI::Event::Connection& connection):
        d_connection(connection)
    {}

    PythonEventConnection(const PythonEventConnection& v):
        d_connection(v.d_connection)
    {}

    bool connected()
    {
        return d_connection.isValid() ? d_connection->connected() : false;
    }

    void disconnect()
    {
        // TODO: Throw on invalid disconnects?
        if (d_connection.isValid())
        {
            d_connection->disconnect();
        }
    }

private:
    CEGUI::Event::Connection d_connection;
};

PythonEventConnection EventSet_subscribeEvent(CEGUI::EventSet* self, const CEGUI::String& name, PyObject* callable)
{
    return PythonEventConnection(self->subscribeEvent(name, PythonEventSubscription(callable)));
}
"""
    )
    eventSet.add_registration_code(
"""
def( "subscribeEvent", &EventSet_subscribeEvent);

    {   // PythonEventConnection

        typedef bp::class_< PythonEventConnection > PythonEventConnection_exposer_t;
        PythonEventConnection_exposer_t PythonEventConnection_exposer = PythonEventConnection_exposer_t( "PythonEventConnection" );
        bp::scope PythonEventConnection_scope( PythonEventConnection_exposer );
        PythonEventConnection_exposer.def( bp::init<>() );
        {

            typedef bool ( PythonEventConnection::*connected_function_type )(  ) ;

            PythonEventConnection_exposer.def(
                "connected"
                , connected_function_type( &PythonEventConnection::connected ) );

        }
        {

            typedef void ( PythonEventConnection::*disconnect_function_type )(  ) ;

            PythonEventConnection_exposer.def(
                "disconnect"
                , disconnect_function_type( &PythonEventConnection::disconnect ) );
        }
    }
"""
    )

    # Exceptions.h
    # handled separately

    # FactoryModule.h
    # not doable in python

    # Font.h
    font = CEGUI_ns.class_("Font", recursive = False)
    font.include()

    # Font_xmlHandler.h
    # not interesting for python

    # FontGlyph.h
    fontGlyph = CEGUI_ns.class_("FontGlyph")
    fontGlyph.include()

    # FontManager.h
    fontManager = CEGUI_ns.class_("FontManager")
    fontManager.include()

    # FormattedRenderedString.
    # todo: is this interesting for python?

    # ForwardRefs.h
    # hell no

    # FreeFunctionSlot.h
    # handled differently elsewhere, see EventSet

    # FreeTypeFont.h
    # not interesting for python

    # FribidiVisualMapping.h
    # not interesting for python

    # FunctorCopySlot.h
    # handled differently elsewhere, see EventSet

    # FunctorPointerSlot.h
    # handled differently elsewhere, see EventSet

    # FunctorReferenceBinder.h
    # handled differently elsewhere, see EventSet

    # FunctorReferenceSlot.h
    # handled differently elsewhere, see EventSet

    # GeometryBuffer.h
    geometryBuffer = CEGUI_ns.class_("GeometryBuffer")
    geometryBuffer.include()

    # GlobalEventSet.h
    globalEventSet = CEGUI_ns.class_("GlobalEventSet")
    globalEventSet.include()

    # GUIContext.h
    guiContext = CEGUI_ns.class_("GUIContext")
    guiContext.include()
    for decl in guiContext.mem_funs("getMouseCursor"):
        if decl.has_const:
            decl.exclude()

    # GUILayout_xmlHandler.h
    # not needed in python

    # Image.h
    autoScaledModeEnum = CEGUI_ns.enum("AutoScaledMode")
    autoScaledModeEnum.include()
    image = CEGUI_ns.class_("Image")
    image.include()

    # ImageCodec.h
    imageCodec = CEGUI_ns.class_("ImageCodec")
    imageCodec.include()

    # ImageManager.h
    imageManager = CEGUI_ns.class_("ImageManager")
    imageManager.include()

    # InputEvent.h
    key = CEGUI_ns.class_("Key")
    key.include()

    mouseButtonEnum = CEGUI_ns.enum("MouseButton")
    mouseButtonEnum.include()

    # Interpolator.h
    interpolator = CEGUI_ns.class_("Interpolator")
    interpolator.include()

    # IteratorBase.h
    # all iterators are sorted later

    # JustifiedRenderedString.h
    # not needed in python

    # KeyFrame.h
    keyFrame = CEGUI_ns.class_("KeyFrame")
    keyFrame.include()

    # LeftAlignedRenderedString.h
    # not needed in python

    # Logger.h
    logger = CEGUI_ns.class_("Logger")
    logger.include()

    loggingLevelEnum = CEGUI_ns.enum("LoggingLevel")
    loggingLevelEnum.include()

    # MemberFunctionSlot.h
    # sorted elsewhere, see EventSet

    # MinibidiVisualMapping.h
    # not needed for python

    # MinizipResourceProvider.h
    # not needed for python

    # MouseCursor.h
    mouseCursor = CEGUI_ns.class_("MouseCursor")
    mouseCursor.include()
    mouseCursor.noncopyable = True

    # NamedElement.h
    namedElement = CEGUI_ns.class_("NamedElement")
    namedElement.include()

    # NamedXMLResourceManager.h
    xmlResourceExistsActionEnum = CEGUI_ns.enum("XMLResourceExistsAction")
    xmlResourceExistsActionEnum.include()

    resourceEventSet = CEGUI_ns.class_("ResourceEventSet")
    resourceEventSet.include()

    # PCRERegexMatcher.h
    # not needed in python

    # PixmapFont.h
    # not needed in python

    # Property.h
    propertyReceiver = CEGUI_ns.class_("PropertyReceiver")
    propertyReceiver.include()

    property = CEGUI_ns.class_("Property")
    property.include()

    # PropertyHelper.h
    propertyHelper = CEGUI_ns.class_("PropertyHelper_wrapper")
    propertyHelper.include()
    propertyHelper.rename("PropertyHelper")

    # PropertySet.h
    propertySet = CEGUI_ns.class_("PropertySet")
    propertySet.include()

    # Quaternion.h
    quaternion = CEGUI_ns.class_("Quaternion")
    quaternion.include()

    # Rect.h
    rectf = CEGUI_ns.class_("Rect<float>")
    rectf.rename("Rectf")
    rectf.include()

    urect = CEGUI_ns.class_("Rect<CEGUI::UDim>")
    urect.rename("URect")
    urect.include()
    # UDim doesn't have the necessary operators for this
    urect.member_function("getIntersection").exclude()
    urect.member_function("isPointInRect").exclude()
    urect.member_function("constrainSizeMin").exclude()
    urect.member_function("constrainSizeMax").exclude()
    urect.member_function("constrainSize").exclude()

    # RefCounted.h
    # handled elsewhere

    # RefexMatcher.h
    regexMatcher = CEGUI_ns.class_("RegexMatcher")
    regexMatcher.include()

    # RenderedString.h
    renderedString = CEGUI_ns.class_("RenderedString")
    renderedString.include()

    # RenderedStringComponent.h
    renderedStringComponent = CEGUI_ns.class_("RenderedStringComponent")
    renderedStringComponent.include()

    # RenderedStringImageComponent.h
    renderedStringImageComponent = CEGUI_ns.class_("RenderedStringImageComponent")
    renderedStringImageComponent.include()

    # RenderedStringTextComponent.h
    renderedStringTextComponent = CEGUI_ns.class_("RenderedStringTextComponent")
    renderedStringTextComponent.include()

    # RenderedStringParser.h
    renderedStringParser = CEGUI_ns.class_("RenderedStringParser")
    renderedStringParser.include()

    # RenderedStringWidgetComponent.h
    renderedStringWidgetComponent = CEGUI_ns.class_("RenderedStringWidgetComponent")
    renderedStringWidgetComponent.include()

    # RenderEffect.h
    renderEffect = CEGUI_ns.class_("RenderEffect")
    renderEffect.include()

    # RenderEffectFactory.h
    renderEffectFactory = CEGUI_ns.class_("RenderEffectFactory")
    renderEffectFactory.include()

    # RenderEffectManager.h
    renderEffectManager = CEGUI_ns.class_("RenderEffectManager")
    renderEffectManager.include()

    # Renderer.h
    quadSplitModeEnum = CEGUI_ns.enum("QuadSplitMode")
    quadSplitModeEnum.include()

    blendModeEnum = CEGUI_ns.enum("BlendMode")
    blendModeEnum.include()

    renderer = CEGUI_ns.class_("Renderer")
    renderer.include()

    # RenderingContext.h
    renderingContext = CEGUI_ns.class_("RenderingContext")
    renderingContext.include()

    # RenderingSurface.h
    CEGUI_ns.enum("RenderQueueID").include()
    renderingSurface = CEGUI_ns.class_("RenderingSurface")
    renderingSurface.include()

    # RenderingWindow.h
    renderingWindow = CEGUI_ns.class_("RenderingWindow")
    renderingWindow.include()

    # RenderQueue.h
    renderQueue = CEGUI_ns.class_("RenderQueue")
    renderQueue.include()

    # RenderTarget.h
    renderTarget = CEGUI_ns.class_("RenderTarget")
    renderTarget.include()

    # ResourceProvider.h
    resourceProvider = CEGUI_ns.class_("ResourceProvider")
    resourceProvider.include()

    # RightAlignedRenderedString.h
    # not needed in python

    # Scheme.h
    scheme = CEGUI_ns.class_("Scheme")
    scheme.include()

    # SchemeManager.h
    schemeManager = CEGUI_ns.class_("SchemeManager")
    schemeManager.include()

    # ScriptModule.h
    scriptModule = CEGUI_ns.class_("ScriptModule")
    scriptModule.include()

    # SimpleTimer.h
    simpleTimer = CEGUI_ns.class_("SimpleTimer")
    simpleTimer.include()

    # Singleton.h
    # handled separately

    # Size.h
    CEGUI_ns.enum("AspectMode").include()
    size = CEGUI_ns.class_("Size<float>")
    size.rename("Sizef")
    size.include()

    usize = CEGUI_ns.class_("Size<CEGUI::UDim>")
    usize.rename("USize")
    usize.include()
    # UDim doesn't have the necessary operators for this
    usize.member_function("clamp").exclude()
    usize.member_function("scaleToAspect").exclude()

    # SlotFunctorBase.h
    # not needed in python

    # String.h
    string = CEGUI_ns.class_("String")
    #string.include()

    # String.h
    stringTranscoder = CEGUI_ns.class_("StringTranscoder")
    # not useful from Python and very hard to wrap
    #stringTranscoder.include()

    # SubscriberSlot.h
    # todo: probably not needed with current setup, should we exclude this?
    subscriberSlot = CEGUI_ns.class_("SubscriberSlot")
    subscriberSlot.include()
    # exclude the constructor that takes function pointer as argument
    for c in subscriberSlot.constructors():
        if len(c.arguments) > 0:
            c.exclude()

    # System.h
    system = CEGUI_ns.class_("System")
    system.include()
    common_utils.excludeAllPrivate(system)
    system.mem_fun("getStringTranscoder").exclude()

    # SystemKeys.h
    systemKeys = CEGUI_ns.class_("SystemKeys")
    systemKeys.include()

    # Texture.h
    texture = CEGUI_ns.class_("Texture")
    texture.include()

    # TextureTarget.h
    textureTarget = CEGUI_ns.class_("TextureTarget")
    textureTarget.include()

    # TextUtils.h
    textUtils = CEGUI_ns.class_("TextUtils")
    textUtils.include()

    # TplWindowFactory.h
    # TplWindowRendererFactory.h
    # TplWRFactoryRegisterer.h
    # python doesn't like templates :-)

    # UDim.h
    udim = CEGUI_ns.class_("UDim")
    udim.include()

    ubox = CEGUI_ns.class_("UBox")
    ubox.include()

    # Vector.h
    vector2f = CEGUI_ns.class_("Vector2<float>")
    vector2f.rename("Vector2f")
    vector2f.include()

    vector3f = CEGUI_ns.class_("Vector3<float>")
    vector3f.rename("Vector3f")
    vector3f.include()

    uvector2 = CEGUI_ns.class_("Vector2<CEGUI::UDim>")
    uvector2.rename("UVector2")
    uvector2.include()

    # Version.h
    # nothing usable for python

    # Vertex.h
    vertex = CEGUI_ns.class_("Vertex")
    vertex.include()

    # WidgetModule.h
    # not needed in python

    # Window.h
    CEGUI_ns.enum("VerticalAlignment").include()
    CEGUI_ns.enum("HorizontalAlignment").include()
    CEGUI_ns.enum("WindowUpdateMode").include()

    window = CEGUI_ns.class_("Window")
    window.include()
    # BidiVisualMapping is excluded from python
    window.mem_fun("getBidiVisualMapping").exclude()
    # python doesn't like void*
    window.mem_fun("setUserData").exclude()
    window.mem_fun("getUserData").exclude()
    # todo: check that getUserData is really a python object
    window.add_declaration_code(
"""
void
Window_setUserData ( ::CEGUI::Window & me, PyObject * data ) {
    me.setUserData ( data );
    }

PyObject *
Window_getUserData ( ::CEGUI::Window & me) {
    void *  data = me.getUserData (  );
    Py_INCREF( (PyObject *) data );     // I'm passing a reference to this object so better inc the ref :)
    return  (PyObject *) data;
    }

typedef bool ( ::CEGUI::Window::*isChild_string_function_type )( const ::CEGUI::String& ) const;
typedef bool ( ::CEGUI::Window::*isChild_ptr_function_type )( const ::CEGUI::Element* ) const;

typedef bool ( ::CEGUI::Window::*isAncestor_string_function_type )( const ::CEGUI::String& ) const;
typedef bool ( ::CEGUI::Window::*isAncestor_ptr_function_type )( const ::CEGUI::Element* ) const;

typedef void ( ::CEGUI::Window::*removeChild_string_function_type )( const ::CEGUI::String& );
typedef void ( ::CEGUI::Window::*removeChild_ptr_function_type )( ::CEGUI::Element* );
"""
    )
    window.add_registration_code("""def ("setUserData", &::Window_setUserData);""")
    window.add_registration_code("""def ("getUserData", &::Window_getUserData);""")

    window.add_registration_code("""def ("isChild", isChild_string_function_type(&::CEGUI::Window::isChild));""")
    window.add_registration_code("""def ("isChild", isChild_ptr_function_type(&::CEGUI::Window::isChild));""")

    window.add_registration_code("""def ("isAncestor", isAncestor_string_function_type(&::CEGUI::Window::isAncestor));""")
    window.add_registration_code("""def ("isAncestor", isAncestor_ptr_function_type(&::CEGUI::Window::isAncestor));""")

    window.add_registration_code("""def ("removeChild", removeChild_string_function_type(&::CEGUI::Window::removeChild));""")
    window.add_registration_code("""def ("removeChild", removeChild_ptr_function_type(&::CEGUI::Window::removeChild));""")

    # WindowFactory.h
    windowFactory = CEGUI_ns.class_("WindowFactory")
    windowFactory.include()

    # WindowFactoryManager.h
    windowFactoryManager = CEGUI_ns.class_("WindowFactoryManager")
    windowFactoryManager.include()

    # WindowManager.h
    windowManager = CEGUI_ns.class_("WindowManager")
    windowManager.include()
    # we do all the layout loading in custom code because we can't use PropertyCallback in python

    windowManager.mem_fun("loadLayoutFromContainer").exclude()
    windowManager.add_declaration_code(
"""
CEGUI::Window*
WindowManager_loadLayoutFromContainer(::CEGUI::WindowManager & me,
    const CEGUI::RawDataContainer& container)
{
    return me.loadLayoutFromContainer(container);
}
"""
    )
    windowManager.add_registration_code(
"""
def ("loadLayoutFromContainer", &::WindowManager_loadLayoutFromContainer,\
        (bp::arg("source")), \
        bp::return_value_policy<bp::reference_existing_object, bp::default_call_policies>());
"""
    )

    windowManager.mem_fun("loadLayoutFromFile").exclude()
    windowManager.add_declaration_code(
"""
CEGUI::Window*
WindowManager_loadLayoutFromFile(::CEGUI::WindowManager & me,
    const CEGUI::String& filename, const CEGUI::String& resourceGroup = "")
{
    return me.loadLayoutFromFile(filename, resourceGroup);
}
"""
    )
    windowManager.add_registration_code(
"""
def ("loadLayoutFromFile", &::WindowManager_loadLayoutFromFile,\
        (bp::arg("filename"), bp::arg("resourceGroup") = ""), \
        bp::return_value_policy<bp::reference_existing_object, bp::default_call_policies>());
"""
    )

    windowManager.mem_fun("loadLayoutFromString").exclude()
    windowManager.add_declaration_code(
"""
CEGUI::Window*
WindowManager_loadLayoutFromString(::CEGUI::WindowManager & me,
    const CEGUI::String& source)
{
    return me.loadLayoutFromString(source);
}
"""
    )
    windowManager.add_registration_code(
"""
def ("loadLayoutFromString", &::WindowManager_loadLayoutFromString,\
        (bp::arg("source")), \
        bp::return_value_policy<bp::reference_existing_object, bp::default_call_policies>());
"""
    )

    # WindowProperties.h
    # not needed in python

    # WindowRenderer.h
    windowRenderer = CEGUI_ns.class_("WindowRenderer", recursive = False)
    windowRenderer.include()

    windowRendererFactory = CEGUI_ns.class_("WindowRendererFactory")
    windowRendererFactory.include()

    # WindowRendererManager.h
    windowRendererManager = CEGUI_ns.class_("WindowRendererManager")
    windowRendererManager.include()

    # WindowRendererModule.h
    # not needed in python

    # WRFactoryRegisterer.h
    # not needed in python

    # XMLAttributes.h
    xmlAttributes = CEGUI_ns.class_("XMLAttributes")
    xmlAttributes.include()

    # XMLHandler.h
    xmlHandler = CEGUI_ns.class_("XMLHandler")
    xmlHandler.include()

    # XMLParser.h
    xmlParser = CEGUI_ns.class_("XMLParser")
    xmlParser.include()

    # XMLSerializer.h
    xmlSerializer = CEGUI_ns.class_("XMLSerializer")
    xmlSerializer.include()

    # all EventArgs
    for cls in CEGUI_ns.classes():
        if cls.name.endswith("EventArgs"):
            cls.include()

    # all Iterator classes
    for cls in CEGUI_ns.classes():
        if not declarations.templates.is_instantiation(cls.name):
            continue

        tplName = declarations.templates.name(cls.name)
        if not tplName.endswith("Iterator"):
            continue

        # now cls is sure to be "Iterator" class
        cls.include()

        # we only allow default parameter-less constructors
        # se lets exclude every constructor that has parameters
        cls.constructors(lambda decl: bool(decl.arguments), allow_empty = True, recursive = False).exclude()

        if tplName.endswith("BaseIterator"):
            import hashlib

            # note: I don't like what I am doing here but it's a lazy solution to
            #       an annoying problem of windows not being able to handle long paths :-/
            #
            #       These are base classes, never instantiated anyways so I think
            #       it doesn't do much except look nasty :-D
            cls.rename("ConstBaseIterator_" + hashlib.md5(cls.name).hexdigest())

        else:
            # we have to make aliases for operator++ and operator--, these are not supported by python
            # instead of operator++, we use next(self) and instead of operator--, we use previous(self)
            cls.add_declaration_code(
"""
void Iterator_next(::CEGUI::%s& t)
{
    t++;
}

void Iterator_previous(::CEGUI::%s& t)
{
    t--;
}
""" % (cls.name, cls.name))
            cls.add_registration_code('def("next", &::Iterator_next)')
            cls.add_registration_code('def("previous", &::Iterator_previous)')

    ### ELEMENTS ###

    # we always skip the Properties files since they are of no use to python users

    # elements/ButtonBase.h
    buttonBase = CEGUI_ns.class_("ButtonBase")
    buttonBase.include()

    # elements/Combobox.h
    combobox = CEGUI_ns.class_("Combobox")
    combobox.include()

    # elements/ComboDropList.h
    comboDropList = CEGUI_ns.class_("ComboDropList")
    comboDropList.include()

    # elements/DefaultWindow.h
    defaultWindow = CEGUI_ns.class_("DefaultWindow")
    defaultWindow.include()

    # elements/DragContainer.h
    dragContainer = CEGUI_ns.class_("DragContainer")
    dragContainer.include()

    # elements/Editbox.h
    editbox = CEGUI_ns.class_("Editbox")
    editbox.include()

    # elements/FrameWindow.h
    frameWindow = CEGUI_ns.class_("FrameWindow")
    frameWindow.include()

    # elements/GridLayoutContainer.h
    gridLayoutContainer = CEGUI_ns.class_("GridLayoutContainer")
    gridLayoutContainer.include()
    # FIXME: Temporary, this should work with transformations
    gridLayoutContainer.mem_fun("mapFromIdxToGrid").exclude()

    # elements/GroupBox.h
    groupBox = CEGUI_ns.class_("GroupBox")
    groupBox.include()

    # elements/HorizontalLayoutContainer.h
    horizontalLayoutContainer = CEGUI_ns.class_("HorizontalLayoutContainer")
    horizontalLayoutContainer.include()

    # elements/ItemEntry.h
    itemEntry = CEGUI_ns.class_("ItemEntry")
    itemEntry.include()

    # elements/ItemListBase.h
    itemListBase = CEGUI_ns.class_("ItemListBase")
    itemListBase.include()
    # needs function pointer as argument
    itemListBase.mem_fun("setSortCallback").exclude()

    # elements/ItemListbox.h
    itemListbox = CEGUI_ns.class_("ItemListbox")
    itemListbox.include()

    # elements/LayoutContainer.h
    layoutContainer = CEGUI_ns.class_("LayoutContainer")
    layoutContainer.include()

    # elements/Listbox.h
    listbox = CEGUI_ns.class_("Listbox")
    listbox.include()
    # we have to handle this separately because CEGUI is nasty and will deallocate this for us unless
    # we remove it from it, also we have to prevent python from destroying the object
    listbox.mem_fun("addItem").exclude()
    listbox.mem_fun("removeItem").exclude()
    listbox.add_declaration_code(
    """
    void
    Listbox_addItem(CEGUI::Listbox& self, PyObject* item)
    {
        CEGUI::ListboxItem* nativeItem = boost::python::extract<CEGUI::ListboxItem*>(boost::python::incref(item));
        // passed from python so don't delete it!
        nativeItem->setAutoDeleted(false);

        self.addItem(nativeItem);
    }

    void
    Listbox_removeItem(CEGUI::Listbox& self, PyObject* item)
    {
        CEGUI::ListboxItem* nativeItem = boost::python::extract<CEGUI::ListboxItem*>(item);
        // don't delete it, python will take care of it
        nativeItem->setAutoDeleted(false);

        self.removeItem(nativeItem);
        boost::python::decref(item);
    }
    """
    )
    listbox.add_registration_code('def ("addItem", &::Listbox_addItem, (bp::arg("item")));')
    listbox.add_registration_code('def ("removeItem", &::Listbox_removeItem, (bp::arg("item")));')

    # elements/ListboxItem.h
    listboxItem = CEGUI_ns.class_("ListboxItem")
    listboxItem.include()

    # elements/ListboxTextItem.h
    listboxTextItem = CEGUI_ns.class_("ListboxTextItem")
    listboxTextItem.include()

    # elements/ListHeader.h
    listHeader = CEGUI_ns.class_("ListHeader")
    listHeader.include()

    # elements/ListHeaderSegment.h
    listHeaderSegment = CEGUI_ns.class_("ListHeaderSegment")
    listHeaderSegment.include()

    # elements/Menubar.h
    menubar = CEGUI_ns.class_("Menubar")
    menubar.include()

    # elements/MenuBase.h
    menuBase = CEGUI_ns.class_("MenuBase")
    menuBase.include()

    # elements/MenuItem.h
    menuItem = CEGUI_ns.class_("MenuItem")
    menuItem.include()

    # elements/MultiColumnList.h
    CEGUI_ns.class_("MCLGridRef").include()
    multiColumnList = CEGUI_ns.class_("MultiColumnList")
    multiColumnList.include()
    # internal private class
    multiColumnList.class_("ListRow").exclude()
    # internal
    multiColumnList.mem_fun("pred_descend").exclude()

    # elements/MultiLineEditbox.h
    multiLineEditbox = CEGUI_ns.class_("MultiLineEditbox")
    multiLineEditbox.include()

    # elements/PopupMenu.h
    popupMenu = CEGUI_ns.class_("PopupMenu")
    popupMenu.include()

    # elements/ProgressBar.h
    progressBar = CEGUI_ns.class_("ProgressBar")
    progressBar.include()

    # elements/PushButton.h
    pushButton = CEGUI_ns.class_("PushButton")
    pushButton.include()

    # elements/RadioButton.h
    radioButton = CEGUI_ns.class_("RadioButton")
    radioButton.include()

    # elements/ScrollablePane.h
    scrollablePane = CEGUI_ns.class_("ScrollablePane")
    scrollablePane.include()

    # elements/Scrollbar.h
    scrollbar = CEGUI_ns.class_("Scrollbar")
    scrollbar.include()

    # elements/ScrolledContainer.h
    scrolledContainer = CEGUI_ns.class_("ScrolledContainer")
    scrolledContainer.include()

    # elements/ScrolledItemListBase.h
    scrolledItemListBase = CEGUI_ns.class_("ScrolledItemListBase")
    scrolledItemListBase.include()

    # elements/SequentialLayoutContainer.h
    sequentialLayoutContainer = CEGUI_ns.class_("SequentialLayoutContainer")
    sequentialLayoutContainer.include()

    # elements/Slider.h
    slider = CEGUI_ns.class_("Slider")
    slider.include()

    # elements/Spinner.h
    spinner = CEGUI_ns.class_("Spinner")
    spinner.include()

    # elements/TabButton.h
    tabButton = CEGUI_ns.class_("TabButton")
    tabButton.include()

    # elements/TabButton.h
    tabControl = CEGUI_ns.class_("TabControl")
    tabControl.include()

    # elements/Thumb.h
    thumb = CEGUI_ns.class_("Thumb")
    thumb.include()

    # elements/Titlebar.h
    titlebar = CEGUI_ns.class_("Titlebar")
    titlebar.include()

    # elements/ToggleButton.h
    toggleButton = CEGUI_ns.class_("ToggleButton")
    toggleButton.include()

    # elements/Tooltip.h
    tooltip = CEGUI_ns.class_("Tooltip", recursive = False)
    tooltip.include()

    # elements/Tree.h
    tree = CEGUI_ns.class_("Tree")
    tree.include()
    # todo: this doesn't work, needs function transformation
    tree.mem_fun("getNextSelectedItemFromList").exclude()

    # elements/TreeItem.h
    treeItem = CEGUI_ns.class_("TreeItem")
    treeItem.include()

    # elements/VerticalLayoutContainer.h
    verticalLayoutContainer = CEGUI_ns.class_("VerticalLayoutContainer")
    verticalLayoutContainer.include()

    ### FALAGARD ###

    # falagard/FalComponentBase.h
    falagardComponentBase = CEGUI_ns.class_("FalagardComponentBase")
    falagardComponentBase.include()

    # falagard/FalDimensions.h
    CEGUI_ns.class_("BaseDim").include()
    CEGUI_ns.class_("OperatorDim").include()
    CEGUI_ns.class_("AbsoluteDim").include()
    CEGUI_ns.class_("ImageDim").include()
    CEGUI_ns.class_("ImagePropertyDim").include()
    CEGUI_ns.class_("WidgetDim").include()
    CEGUI_ns.class_("UnifiedDim").include()
    CEGUI_ns.class_("FontDim").include()
    CEGUI_ns.class_("PropertyDim").include()
    CEGUI_ns.class_("Dimension").include()
    CEGUI_ns.class_("ComponentArea").include()

    # falagard/FalEnums.h
    CEGUI_ns.enum("DimensionType").include()
    CEGUI_ns.enum("VerticalFormatting").include()
    CEGUI_ns.enum("HorizontalFormatting").include()
    CEGUI_ns.enum("VerticalTextFormatting").include()
    CEGUI_ns.enum("HorizontalTextFormatting").include()
    CEGUI_ns.enum("FontMetricType").include()
    CEGUI_ns.enum("DimensionOperator").include()
    CEGUI_ns.enum("FrameImageComponent").include()
    CEGUI_ns.enum("ChildEventAction").include()

    # falagard/EventAction.h
    eventAction = CEGUI_ns.class_("EventAction")
    eventAction.include()

    # falagard/FalEventLinkDefinition.h
    eventLinkDefinition = CEGUI_ns.class_("EventLinkDefinition")
    eventLinkDefinition.include()

    # falagard/FalFrameComponent.h
    frameComponent = CEGUI_ns.class_("FrameComponent")
    frameComponent.include()

    # falagard/FalImageryComponent.h
    imageryComponent = CEGUI_ns.class_("ImageryComponent")
    imageryComponent.include()

    # falagard/FalImagerySection.h
    imagerySection = CEGUI_ns.class_("ImagerySection")
    imagerySection.include()

    # falagard/FalLayerSpecification.h
    layerSpecification = CEGUI_ns.class_("LayerSpecification")
    layerSpecification.include()

    # falagard/FalNamedArea.h
    namedArea = CEGUI_ns.class_("NamedArea")
    namedArea.include()

    # TypedProperty.h
    # falagard/FalPropertyDefinition.h
    # falagard/FalPropertyLinkDefinition.h
    for cls in CEGUI_ns.classes():
        if not declarations.templates.is_instantiation(cls.name):
            continue

        tplName = declarations.templates.name(cls.name)

        desired_classes = ["TypedProperty", "FalagardPropertyBase", "PropertyDefinition", "PropertyLinkDefinition"]

        for desired_class in desired_classes:
            if tplName.startswith(desired_class):
                cls.include()

                if tplName.startswith("TypedProperty"):
                    # gccxml has issues with the T() default arg in the ctor
                    # since users are unlikely to construct TypedProperty from
                    # Python we can just exclude the ctors
                    cls.constructors().exclude()

    # falagard/FalPropertyDefinitionBase.h
    propertyDefinitionBase = CEGUI_ns.class_("PropertyDefinitionBase")
    propertyDefinitionBase.include()

    # falagard/FalPropertyInitialiser.h
    propertyInitialiser = CEGUI_ns.class_("PropertyInitialiser")
    propertyInitialiser.include()

    # falagard/FalSectionSpecification.h
    sectionSpecification = CEGUI_ns.class_("SectionSpecification")
    sectionSpecification.include()

    # falagard/FalStateImagery.h
    stateImagery = CEGUI_ns.class_("StateImagery")
    stateImagery.include()

    # falagard/FalTextComponent.h
    textComponent = CEGUI_ns.class_("TextComponent")
    textComponent.include()

    # falagard/FalWidgetComponent.h
    widgetComponent = CEGUI_ns.class_("WidgetComponent")
    widgetComponent.include()

    # falagard/FalWidgetLookFeel.h
    widgetLookFeel = CEGUI_ns.class_("WidgetLookFeel")
    widgetLookFeel.include()

    # falagard/FalWidgetLookManager.h
    widgetLookManager = CEGUI_ns.class_("WidgetLookManager")
    widgetLookManager.include()

    # falagard/FalXMLEnumHelper.h
    falagardXMLHelper = CEGUI_ns.class_("FalagardXMLHelper_wrapper")
    falagardXMLHelper.include()
    falagardXMLHelper.rename("FalagardXMLHelper")

    # todo: hack fixes
    # taken from python ogre, causes AttributeError at import if not excluded
    for cls in CEGUI_ns.classes():
        try:
            cls.variable("EventNamespace").exclude()
        except:
            pass
    ## turns out that in SOME classes this also fails registration (Combodroplist for example)
    for cls in CEGUI_ns.classes():
        try:
            cls.variable("WidgetTypeName").exclude()
        except:
            pass

    # taken from python ogre
    ## lets work around a bug in GCCXMl - http://language-binding.net/pygccxml/design.html#patchers
    draws = mb.mem_funs("draw")   # find all the draw functions
    for draw in draws:
        for arg in draw.arguments:
            if arg.default_value == "0ffffffff":
                arg.default_value = "0xffffffff"

    for cls in CEGUI_ns.classes():
        if cls.name.startswith("Singleton"):
            cls.include()

        if cls.name.startswith("NamedXMLResourceManager"):
            cls.include()
            # WORKAROUND: would not compile on Windows, pyplusplus substitutes the internal _Rb_tree_iterator
            cls.mem_fun("destroyObject").exclude()

    # no need for this function, just use getSingleton
    mb.mem_funs("getSingletonPtr").exclude()

    # include workaround functions from the CEGUI namespace
    CEGUI_ns.class_("Workarounds").include();

    # at no point should you need any private methods
    common_utils.excludeAllPrivate(CEGUI_ns)

def configureExceptions(mb):
    exception = mb.namespace("CEGUI").class_("Exception")
    exception.include()
    exception.mem_funs().exclude()
    exception.mem_fun("setStdErrEnabled").include()
    exception.mem_fun("isStdErrEnabled").include()
    exception.translate_exception_to_string("PyExc_RuntimeError", "exc.what()")

def generate():
    ### disable unnecessary warnings
    # can't pass function pointer
    messages.disable(messages.W1004)
    # operator not supported
    messages.disable(messages.W1014)
    # py++ will create a wrapper
    messages.disable(messages.W1023, messages.W1025, messages.W1026, messages.W1027, messages.W1031)
    # static pointer member can't be exported
    messages.disable(messages.W1035)
    # immutable pointer can't be exposed
    messages.disable(messages.W1036)
    # pointer to function can't be exposed
    messages.disable(messages.W1036, messages.W1037)
    # can't be overridden in python
    messages.disable(messages.W1049)

    mb = common_utils.createModuleBuilder("python_CEGUI.h", ["CEGUIBASE_EXPORTS"])
    CEGUI_ns = mb.global_ns.namespace("CEGUI")
    # first thing to do - converters!
    # !!! They have to be registered first, otherwise it will all fail horribly !!!
    common_utils.addStringConverters(mb)
    common_utils.addSupportForString(mb)

    filterDeclarations(mb)
    configureExceptions(mb)

    common_utils.setDefaultCallPolicies(CEGUI_ns)

    ## add additional version information to the module to help identify it correctly
    # todo: this should be done automatically
    common_utils.addVersionInfo(mb, PACKAGE_NAME, PACKAGE_VERSION)

    # Creating code creator. After this step you should not modify/customize declarations.
    mb.build_code_creator(module_name = MODULE_NAME, doc_extractor = common_utils.createDocumentationExtractor())

    common_utils.writeModule(mb, OUTPUT_DIR)

if __name__ == "__main__":
    common_utils.verbose_generate("CEGUIBase", generate)



import sys

CEGUI_BASEDIR = "C:\\Users\\Admin\\Devel\\CEGUI\\cegui_mk2"
is_64bits = sys.maxsize > 2**32
BOOST_BASEDIR = "C:\\Program Files (x86)\\boost\\boost_1_46_1" if is_64bits else "C:\\Program Files\\boost\\boost_1_46_1"

from distutils.core import setup, Extension
from distutils.sysconfig import get_python_inc

import os
from glob import *

global_defines = []# [("CEGUI_STATIC", "1")]
# turn on exceptions for MSVC
global_extra_compile_args = ["/EHsc"]
global_include_dirs = [get_python_inc(plat_specific = True), "../", "../../../../../include", "../../../../../../build/cegui/include", BOOST_BASEDIR]
global_library_dirs = [CEGUI_BASEDIR + "/build/lib", BOOST_BASEDIR + "/lib"]
# Windows is special and picks the libraries magically!
global_libraries = []

PyCEGUI_sources = glob(os.path.join("..", "output", "CEGUI", "*.cpp"))
PyCEGUIOpenGLRenderer_sources = glob(os.path.join("..", "output", "CEGUIOpenGLRenderer", "*.cpp"))
PyCEGUIOgreRenderer_sources = glob(os.path.join("..", "output", "CEGUIOgreRenderer", "*.cpp"))
PyCEGUINullRenderer_sources = glob(os.path.join("..", "output", "CEGUINullRenderer", "*.cpp"))

setup(
    name = "PyCEGUI",
    version = "0.8",
    description = "Python bindings for CEGUI library",
    long_description =
"""Crazy Eddie's GUI System is a free library providing windowing
and widgets for graphics APIs / engines where such functionality
is not natively available, or severely lacking. The library is
object orientated, written in C++, and targeted at games developers
who should be spending their time creating great games, not building GUI sub-systems.

note: For Linux and MacOSX packages, see http://www.cegui.org.uk, we provide them
      in SDKs. Distutils package is only provided for Windows since it's hard to
      install the binding there as it involves lots of wrappers and nasty tricks.
      Shame on you Windows!""",
    author = "CEGUI team",
    author_email = "team@cegui.org.uk",
    #maintainer = "Martin Preisler", # authors get shadowed by this
    #maintainer_email = "preisler.m@gmail.com",
    url = "http://www.cegui.org.uk",
    license = "MIT",
    platforms = ["Windows", "Linux", "MacOSX"],
    
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: MacOS X",
        "Environment :: Win32 (MS Windows)",
        "Environment :: X11 Applications",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: C++"
        "Topic :: Games/Entertainment",
        "Topic :: Multimedia :: Graphics",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: User Interfaces",
        "Topic :: Software Development :: Widget Sets",
    ],
    
    packages = [
        "PyCEGUI" # nasty nasty wrapper for a nasty nasty OS
    ],
    
    # this doesn't work as I expected so I am doing data files further down
    #package_data = { "PyCEGUI/datafiles": [CEGUI_BASEDIR + "/datafiles/*"] },
        
    ext_modules = [
        Extension(
            "PyCEGUI.PyCEGUI",
            PyCEGUI_sources,
            define_macros = global_defines,
            extra_compile_args = global_extra_compile_args,
            include_dirs = global_include_dirs + ["../output/CEGUI"],
            library_dirs = global_library_dirs,
            libraries = ["CEGUIBase"] + global_libraries,
        ),
        Extension(
            "PyCEGUIOpenGLRenderer",
            PyCEGUIOpenGLRenderer_sources,
            define_macros = global_defines,
            extra_compile_args = global_extra_compile_args,
            include_dirs = global_include_dirs + ["../output/CEGUIOpenGLRenderer"],
            library_dirs = global_library_dirs,
            libraries = ["CEGUIBase", "CEGUIOpenGLRenderer"] + global_libraries,
        ),
        # no ogre for now, I will use SDK binaries later for this
        #Extension(
        #    "PyCEGUIOgreRenderer",
        #    PyCEGUIOgreRenderer_sources,
        #    define_macros = global_defines,
        #    extra_compile_args = global_extra_compile_args,
        #    include_dirs = global_include_dirs + ["../output/CEGUIOgreRenderer"],
        #    library_dirs = global_library_dirs,
        #    libraries = ["CEGUIBase", "CEGUIOgreRenderer"] + global_libraries,
        #),
        Extension(
            "PyCEGUINullRenderer",
            PyCEGUINullRenderer_sources,
            define_macros = global_defines,
            extra_compile_args = global_extra_compile_args,
            include_dirs = global_include_dirs + ["../output/CEGUINullRenderer"],
            library_dirs = global_library_dirs,
            libraries = ["CEGUIBase", "CEGUINullRenderer"] + global_libraries,
        ),
    ],
    
    # the first string is directory where the files should go
    # - leave empty for C:/Python26 for example
    data_files = [
        # we have to bundle CEGUIBase.dll, CEGUIOpenGLRenderer.dll, etc...
        ("Lib/site-packages/PyCEGUI",
        # this is obviously a workaround, I would be happy to hear what the clean
        # solution should look like
            [
                BOOST_BASEDIR + "/lib/boost_python-vc90-mt-1_46_1.dll",
                
                CEGUI_BASEDIR + "/build/bin/CEGUIBase.dll",
                CEGUI_BASEDIR + "/build/bin/CEGUIOpenGLRenderer.dll",
                CEGUI_BASEDIR + "/build/bin/CEGUINullRenderer.dll",
                CEGUI_BASEDIR + "/build/lib/CEGUICoreWindowRendererSet.dll",
                CEGUI_BASEDIR + "/build/lib/CEGUIFreeImageImageCodec.dll",
                CEGUI_BASEDIR + "/dependencies/bin/freeimage.dll",
                CEGUI_BASEDIR + "/build/lib/CEGUIExpatParser.dll",
            ]
        ),
        
        # distutils doesn't allow to bundle folders (or to be precise: I have no idea how to do that)
        # therefore I do this the ugly way!
        ("Lib/site-packages/PyCEGUI/datafiles/animations",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/animations", "*")),
        ),
        ("Lib/site-packages/PyCEGUI/datafiles/configs",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/configs", "*")),
        ),
        ("Lib/site-packages/PyCEGUI/datafiles/fonts",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/fonts", "*")),
        ),
        ("Lib/site-packages/PyCEGUI/datafiles/imagesets",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/imagesets", "*")),
        ),
        ("Lib/site-packages/PyCEGUI/datafiles/layouts",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/layouts", "*")),
        ),
        ("Lib/site-packages/PyCEGUI/datafiles/looknfeel",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/looknfeel", "*")),
        ),
        ("Lib/site-packages/PyCEGUI/datafiles/lua_scripts",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/lua_scripts", "*")),
        ),
        ("Lib/site-packages/PyCEGUI/datafiles/schemes",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/schemes", "*")),
        ),
        ("Lib/site-packages/PyCEGUI/datafiles/xml_schemas",
            glob(os.path.join(CEGUI_BASEDIR + "/datafiles/xml_schemas", "*")),
        )
    ]
)
